/**
 * @file model.h
 * @author Tom Molesworth <tom@entitymodel.com>
 * @date 17/11/12 12:03:08
 *
 * $Id$
 */
#ifndef ENTITYMODEL__MODEL_H__
#define ENTITYMODEL__MODEL_H__

#include "Author.h"
#include "Book.h"
#include "Library.h"
#include "LibraryItem.h"

#endif

