/**
 * @file library.h
 * @author Tom Molesworth <tom@entitymodel.com>
 * @date 17/11/12 12:17:53
 *
 * $Id$
 */

#include <string>

namespace Model {

class Book;

class Library {
public:
	Library() {
	}

virtual	~Library() { }

private:
};

};


